#import "CocoaZonicController.h"
#import "ZonicKRM.h"


@interface NSObject (ZonicDelegate)
- (void) zonicKRMCompleted:(ZKRMResult*)zonicResult;
@end

@interface CocoaZonicController (Private)
- (void) doIntegratedPurchase:(BOOL)usingModalDialog;
@end

@implementation CocoaZonicController

static pascal OSStatus HandleCarbonEvent(EventHandlerCallRef theHandler, EventRef theEvent, void *userData)
{	
	OSStatus returnError = eventNotHandledErr;
	id krmDelegate = (id)userData;
	UInt32 eventClass = GetEventClass(theEvent);
	UInt32 eventKind  = GetEventKind(theEvent);
	
	if (eventClass == kEventClassZKRM && eventKind == kEventZKRMCompleted)
	{
		ZKRMResult		*theResult;
	
		// Process a KRM result
		//
		// Delivered asynchronously after a successful call to BeginModelessKRM.
		GetEventParameter(theEvent, kEventParamZKRMResult, typeUInt32, NULL, sizeof(theResult), NULL, &theResult);
		if (theResult != NULL)
		{
			if ((krmDelegate != nil) && ([krmDelegate respondsToSelector:@selector(zonicKRMCompleted:)] == YES))
			{
				[krmDelegate zonicKRMCompleted:theResult];
			}
			DisposeKRMResult(&theResult);
			returnError = noErr;
		}
	}
	
	return returnError;
}

- (IBAction)startIntegratedModelessPurchase:(id)sender
{
	[self doIntegratedPurchase:NO];
}

- (IBAction)startIntegratedModalPurchase:(id)sender
{
	[self doIntegratedPurchase:YES];
}

- (void) doIntegratedPurchase:(BOOL)usingModalDialog
{
	static ZKRMParameters	theParameters;
	ZKRMModuleStatus		theStatus;

	// Prepare the KRM parameters
	//
	// The following information should be customised:
	//
	//		http://www.mywebsite.invalid/*.html		URLs to your web site
	//		UC9										Your vendor code    (as held by Kagi)
	//		KART_TEST_COMPLEX_ACG_UC9				Your product name   (as held by Kagi)
	//		"My Product"							Your product name   (as seen by user)
	//		<price>									Pricing information (e.g., relative to a US $ price)
	//		http://order.kagi.com/?UC9				URL to your store   (as held by Kagi)
	//		My_Affiliate							An affiliate code   (as held by Kagi)
	//
	InitKRMParameters(&theParameters);
	
	theParameters.moduleLanguage  = kZKRMLanguageSystem;
	theParameters.moduleOptions   = kZKRMOptionDefault;
	theParameters.moduleUserName  = NULL;
	theParameters.moduleUserEmail = NULL;
	theParameters.productInitXML  = CFSTR(
					"<krmData vendorID=\"UC9\">"
					"<transactionFailureURL>http://www.mywebsite.invalid/MyTransactionFailurePage.html</transactionFailureURL>"
					"<connectionFailureURL>http://www.mywebsite.invalid/MyConnectionFailurePage.html</connectionFailureURL>"
					"<products>"
						"<product partNo=\"\" dbName=\"KART_TEST_COMPLEX_ACG_UC9\" supplierSKU=\"n/a\">"
							"<displayName lang=\"en\">My Product</displayName>"
							"<price currency=\"AUD\" basePrice=\"100.00\" baseCurrency=\"USD\"/>"
							"<price currency=\"CAD\" basePrice=\"100.00\" baseCurrency=\"USD\"/>"
							"<price currency=\"EUR\" basePrice=\"100.00\" baseCurrency=\"USD\"/>"
							"<price currency=\"GBP\" basePrice=\"100.00\" baseCurrency=\"USD\"/>"
							"<price currency=\"JPY\" basePrice=\"100.00\" baseCurrency=\"USD\"/>"
							"<price currency=\"USD\">100.00</price>"
						"</product>"
					"</products>"
					"</krmData>");

	theParameters.productStoreURL  = CFSTR("http://order.kagi.com/?UC9");
	theParameters.productTFYP      = CFSTR("Thanks for your purchase.");
	theParameters.productPO        = CFSTR("Custom string to use as a Purchase Order number");
	theParameters.productAffiliate = CFSTR("My_Affiliate");

	// Invoke the KRM
	if (usingModalDialog == YES)
	{
		ZKRMResult *myResult;
		theStatus = BeginModalKRM(&theParameters, &myResult);
		if (theStatus == kZKRMModuleNoErr)
		{
			[self zonicKRMCompleted:myResult];

			// Release the result
			DisposeKRMResult(&myResult);			
		}		
	}
	else
	{
		// We subscribe to KRM event completed to ensure we get called back when the modeless dialog is dismissed.
		// If we can't subscribe, then the dialog's result will be useless to us, so don't bother.
		const EventTypeSpec kCarbonEvents[] = {{kEventClassZKRM, kEventZKRMCompleted}};
		if (InstallApplicationEventHandler(HandleCarbonEvent, 1, kCarbonEvents, self, &mKRMEventHandler) != noErr)
		{
			NSLog(@"Failed to subscribe to KRM completed event. Can't start modeless dialog.");
		}
		else
		{
			theStatus = BeginModelessKRM(&theParameters);
			if (theStatus != kZKRMModuleNoErr)
			{
				NSLog(@"KRM Failed to start modeless.");
				if (mKRMEventHandler != NULL)
				{
					RemoveEventHandler(mKRMEventHandler);	
					mKRMEventHandler = NULL;
				}
			}
		}
	}
}

- (void) zonicKRMCompleted:(ZKRMResult *)zonicResult
{		
	// Dialog is done, so remove our hooks
	if (mKRMEventHandler != NULL)
	{
		RemoveEventHandler(mKRMEventHandler);	
		mKRMEventHandler = NULL;
	}
	
	if (zonicResult->moduleStatus != kZKRMModuleNoErr)
	{
		NSLog(@"Zonic canceled or an unexpected error occurred. Don't register!");
	}
	else
	{
		NSString* newUserName = (NSString*) zonicResult->acgUserName;
		NSString* newRegCode = (NSString*) zonicResult->acgRegCode;
		NSLog(@"Zonic success! Register user %@ with code %@", newUserName, newRegCode);
	}
}

@end
